#
# GanymedeRemote.pl
#
# Copyright 1998-2010 The University of Texas at Austin
#
# Ganymede: http://www.arlut.utexas.edu/gash2/
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA
#
#                  --------------------  
#
# This script is designed to execute on a Windows NT Primary Domain Controller.
#
# The ntsambaBuild script in the Ganymede userKit will create a text file
# that contains user and group account synchronization information.  This
# script is designed to be called via rsh (or ssh, if you can find an
# affordable ssh service for NT), with this file provided to STDIN.
#
# The file sent to stdin for this script is formatted as follows,
# with any colons in fields (such as in the password field) escaped
# with a leading back-slash.
#
# [Create/Update]
# username:oldusername:pass:fullname:comment
# username:oldusername:pass:fullname:comment
# ...
# [Inactivate]
# username
# username
# ..
# [Delete]
# username
# username
# ..
# [Create/Update Groups]
# groupname:oldgroupname:userlist:comment
# groupname:oldgroupname:userlist:comment
# ...
# [Inactivate Groups]
# groupname
# groupname
# ..
# [Delete Groups]
# groupname
# groupname
# <eof>
# 
# Jonathan Abbey
# 3 August 1998
#
# Bob Johnson
# 25 June 1999
# -added code to check for pre-existing user accounts
#  if account already exsists on NT, then update info.
#
# Jonathan Abbey
# 4 April 2000
# - added code for group support
#
# Jonathan Abbey
# 25 January 2001
# - prepared for release to the world
#
##############################################################

# SEE: http://theory.uwinnipeg.ca/CPAN/data/libwin32/NetAdmin/NetAdmin.html
# for documentation on Win32::NetAdmin PERL module.

use Win32::NetAdmin;

# SEE: http://www.roth.net/perl/adminmisc/
# for documentation on Win32::AdminMisc PERL module.

use Win32::AdminMisc;

use Win32::Eventlog;

# don't buffer stdout, so we get full debug info

$| = 1;

# ----------------------------------------------------------------------
#
# define our constants
#
# ----------------------------------------------------------------------

# You'll need to edit this to set the WINS name of your NT Primary
# Domain Controller.  Remember that in Perl you have to use put two
# backslashes in a string for each backslash you actually want to have.

$server = "\\\\MYSERVERNAME";

# Where are we going to write out our debug information?

$debugLog = "c:\\ganymede\\report.txt";

# These two variables probably don't need to be changed

$passwordAge = 0;
$privilege = USER_PRIV_USER;

# Ganymede with the userKit doesn't give us a per-user home directory for
# our Domain.. just use a reasonable default.

$homeDir = 'c:\\';

# set the initial flags we want for users sync'ed from ganymede.  we
# set UF_PASSWD_NOTREQD here so that the Win32 API doesn't attempt to
# force the user to reset their password on first login, even after we
# set the password.
#
# In reality, we do all password setting via this script (hence
# UF_PASSWD_CANT_CHANGE), and if the password is blank, we'll
# disable the account.

$flags = UF_NORMAL_ACCOUNT | UF_SCRIPT | UF_PASSWD_CANT_CHANGE | UF_PASSWD_NOTREQD;
$updateflags = 0;
$scriptPath = 'c:\\';

$debug = 1;

##############################################################

# create event log

LogEvent ("Started","1","Ganymede started");

$debug && print "GanymedeRemote Perl Script on NT running\n";

$mode = -1;

open(OUT, ">c:\\perladmin\\report.txt") || die ("Couldn't open report.txt");

while (<STDIN>) {

  chomp $_;

  if ($_ eq "[Create/Update]") {
    $mode = 1;
    next;
  } elsif ($_ eq "[Inactivate]") {
    $mode = 2;
    next;
  } elsif ($_ eq "[Delete]") {
    $mode = 3;
    next;
  } elsif ($_ eq "[Create/Update Groups]") {
    $mode = 4;
    next;
  } elsif ($_ eq "[Inactivate Groups]") {
    $mode = 5;
    next;
  } elsif ($_ eq "[Delete Groups]") {
    $mode = 6;
    next;
  } elsif ($_ eq "[EOF]") {
    last;
  }

  # Create/update

  if ($mode == 1) {
    ($username, $oldname, $pass, $fullname, $comment) = colonsplitter($_);

#    $debug && print "Getting a user create/update attempt.. $username:$oldname:$pass:$fullname:$comment\n";

    if ($oldname ne "") {
      if (renameUser($username, $oldname)) {
	
	$debug && print "Successfully renamed $oldname to $username\n";

	if (updateUser($username, $pass, $comment)) {
#	  print "Successfully updated renamed user $username\n";
	  next;
	}
      } else {
	$debug && print "Could not rename $oldname to $username\n";
	next;
      }
    }

    if (updateUser($username, $pass, $comment)) {
#      $debug && print "Successfully updated $username\n";
      next;
    } else {
      $debug && print "Could not create/update $username\n";
      print OUT "Could not create/update $username\n";
      next;
    }
  }

  # Inactivate User

  if ($mode == 2) {
    /(\S*)/;
    $username = $1;
    
    inactivateUser($username);
    next;
  }

  # Delete User

  if ($mode == 3) {
    /(\S*)/;
    $username = $1;
    
    if (deleteUser($username)) {
      $debug && print "Deleted user $username\n";
      print OUT "Deleted user $username\n";
    } else {
      $debug && print "Could not delete user $username\n";
      print OUT "Could not delete user $username\n";
    }

    next;
  }

  # Sync a Group

  if ($mode == 4) {
    ($groupname, $oldname, $groupMembers, $description) = colonsplitter($_);
#    $debug && print "Getting a group update attempt: $groupname:$oldname:$groupMembers:$description\n";
    &syncGroup($groupname, $oldname, $groupMembers, $description);

    next;
  }

  # Inactivate Group

  if ($mode == 5) {
    /(\S*)/;
    $groupname = $1;
    
    if (emptyGroup($groupname)) {
      $debug && print "Inactivated group $groupname\n";
      print OUT "Inactivated group $groupname\n";
    } else {
      $debug && print "Could not inactivate group $groupname\n";
      print OUT "Could not inactivate group $groupname\n";
    }

    next;
  }

  # Delete Group

  if ($mode == 6) {
    /(\S*)/;
    $groupname = $1;
    
    if (deleteGroup($groupname)) {
      $debug && print "Deleted group $groupname\n";
      print OUT "Deleted group $groupname\n";
    } else {
      $debug && print "Could not delete group $groupname\n";
      print OUT "Could not delete group $groupname\n";
    }

    next;
  }

  print "Can't process line.. don't support mode $mode yet\n";
  next;
}

close OUT;
# create event log
LogEvent ("Completed","1","Ganymede Completed");

exit(0);

######################################################################
#
# This subroutine takes a username, password and comment (gcos-style)
# field and does whatever it takes to get the user account updated.
#
# This routine can also be used to re-activate a user that was
# previously inactivated.
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub updateUser {
  my ($username, $pass, $comment) = @_;
  my ($tmp_fullname, $tmp_pass, $tmp_age, $tmp_privs, $tmp_homedir);
  my ($tmp_comment, $tmp_flags, $tmp_scriptpath);

  # make sure any comment field we set includes the Ganymede keyword

  if ($comment !~ /Ganymede/) {
    $comment = "Ganymede [" . $comment . "]";
  }

  if (Win32::NetAdmin::UsersExist("", $username)) {

    # update the user's stuff if we have the password for them

    if ($pass ne "") {
      if (Win32::AdminMisc::UserGetAttributes($server, $username, $tmp_fullname,
					      $tmp_pass, $tmp_age,
					      $tmp_privs, $tmp_homedir,
					      $tmp_comment, $tmp_flags,
					      $tmp_scriptpath)) {

	# don't set the password unless the account's comment already
	# has the 'Ganymede' keyword in the user account comment, for
	# security
	
	if ($tmp_comment =~ /Ganymede/) {
	  $success = Win32::AdminMisc::UserSetMiscAttributes($server, 
							     $username,
							     USER_FLAGS => $flags,
							     USER_PASSWORD => $pass,
							     USER_FULL_NAME=>$fullname,
							     USER_COMMENT=>$comment,
							     USER_PASSWORD_EXPIRED=>0);
	
	  $debug && print "Updated/reactivated user $username.\n";
	  print OUT "Updated/reactivated user $username.\n";
	} else {
	  print "*** ERROR, couldn't update information for user account $username.\n";
	  print "***        comment field does not include 'Ganymede'.. attempt to \n";
	  print "***        override a local account with one created in Ganymede?\n";

	  print OUT "Error, couldn't update info for local account $username.\n";
	}
      }
    } else {
      # we don't have password information, don't try updating this
      # pre-existing account

      return 1;
    }

#    $debug && print "Updating user $username\n";
#    print OUT "Updating user $username\n";

    if (!$success) {
	$error = Win32::FormatMessage(Win32::GetLastError());
        print ">Could not update user $username\n";
        print OUT ">Could not update user $username\n";
	print ">$error\n";
	print OUT ">$error\n";
    }
  } else {
    
    # new user

    $debug && print "Creating user $username\n";
    
    if ($pass ne "") {    
      $success = Win32::NetAdmin::UserCreate($server, $username, $pass, $passwordAge, 
					     $privilege, $homeDir,
					     $comment, $flags, $scriptPath);
    } else {
      $success = Win32::NetAdmin::UserCreate($server, $username, $pass, $passwordAge, 
					     $privilege, $homeDir,
					     $comment, $flags | UF_ACCOUNTDISABLE, $scriptPath);
    }

    if (!$success) {
      $error = Win32::FormatMessage(Win32::GetLastError());
      print "Could not create user $username\n";
      print OUT "Could not create user $username\n";
      print "\t$error\n";
      print OUT "\t$error\n";
    }
    
    $success = Win32::NetAdmin::LocalGroupAddUsers('', 'Users', $username);
    
    if (!$success) {
      $error = Win32::FormatMessage(Win32::GetLastError());
      print "Could not add user $username to Local User Group.\n";
      print OUT "Could not add user $username to Local User Group.\n";
      print "\t$error\n";
      print OUT "\t$error\n";
    }
  }
  
  return $success;
}

######################################################################
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub renameUser {
  my ($username, $oldusername) = @_;

  if (Win32::NetAdmin::UsersExist("", $username)) {
    
    # target name already exists
    
    return 0; 
  } else {
 
    # create event log
    LogEvent ("rename","2","$oldusername renamed to $username");

    # rename it
  
    return Win32::AdminMisc::RenameUser('', $oldusername, $username);
  }
}

######################################################################
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub inactivateUser {
  my ($username) = @_;
  my $inactiveFlags;

  if (!Win32::NetAdmin::UsersExist($server, $username)) {
    # no such user, don't bother trying to inactivate
    return 1; 
  } else {
    $inactiveFlags = $flags | UF_ACCOUNTDISABLE;
    
    if (Win32::AdminMisc::UserSetMiscAttributes($server, $username,
						USER_FLAGS => $inactiveFlags)) {
      LogEvent ("inactivated","3","$username inactivated");
      $debug && print "Inactivated user $username.\n";
      print OUT "Inactivated user $username.\n";
      
      return 1;
    } else {
      $debug && print "Couldn't inactivate user $username.\n";
      print OUT "Couldn't inactivate user $username.\n";
    
      return 0;
    }
  }
}

######################################################################
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub deleteUser {
  my ($username) = @_;

  # create event log
  LogEvent ("delete","4","$username deleted");

  return Win32::NetAdmin::UserDelete($server, $username);
}

######################################################################
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub syncGroup {
  my ($groupname, $oldname, $groupMembers, $description) = @_;
  my @members, $person, $comment, @oldmembers, @addmembers, @killmembers;
  my %userhash, %currentUsers, $deaduser;

  @members = split /,/, $groupMembers;
  
# $debug && print "Syncing group $groupname with users: @members\n";

  # build up %userhash with who we want to have in the group

  foreach $person (@members) {
    $userhash{$person} = 1;
  }

  # if we need to rename the group, we have to delete it and recreate it

  if ($oldname ne "") {
    if (Win32::NetAdmin::GroupGetAttributes($server, $groupname, $comment)) {
      if ($comment =~ /Ganymede/) {
	Win32::NetAdmin::GroupDelete($server, $groupname);
      } else {
	$debug && print "Can't delete old groupname $groupname.. not created by Ganymede.\n";
	print OUT "Can't delete old groupname $groupname.. not created by Ganymede.\n";
      }
    }
  }

  # this will fail if we just deleted it, so we'll fall through to
  # creating it in the else clause
  
  if (Win32::NetAdmin::GroupGetAttributes($server, $groupname, $comment)) {
#   $debug && print "Found pre-existing group $groupname.\n";
    
    if ($comment !~ /Ganymede/) {
      $debug && print "Group $groupname was not created by Ganymede.. will not modify.\n";
      print OUT "Group $groupname was not created by Ganymede.. will not modify.\n";
      return 0;
    }

    # if the comment changed, set it

    if ($comment ne $description) {
      Win32::NetAdmin::GroupSetAttributes($server, $groupname, $description);
    }
    
    # get the current users in the group

    @oldmembers = ();

#   $debug && print "Before GroupGetMembers on $groupname, server $server, oldmembers = @oldmembers\n";
    
    if (Win32::NetAdmin::GroupGetMembers($server, $groupname, \@oldmembers)) {
#     $debug && print "Group $groupname has current members: @oldmembers\n";

      # any members in the group that we don't want there, we add to
      # @killmembers

      @killmembers = ();
      
      foreach $person (@oldmembers) {
	if (defined $userhash{$person}) {
	  delete $userhash{$person};
	} else {
	  push @killmembers, $person;
#	  $debug && print "*** need to delete $person\n";
	}
      }
      
      if ($#killmembers >= 0)
	{
	  $debug && print "Attempting to delete users @killmembers from group $groupname.\n";

	  foreach $deaduser (@killmembers) {
#	    $debug && print "Attempting to delete user $deaduser from group $groupname.\n";
	  
	    if (Win32::NetAdmin::GroupDeleteUsers($server, $groupname, $deaduser))
	      {
		$debug && print "Successfully removed $deaduser from group $groupname.\n";
		print OUT "Successfully removed $deaduser from group $groupname.\n";
	      }
	    else
	      {
		$debug && print "Could not remove $deaduser from group $groupname.\n";
		print OUT "Could not remove $deaduser from group $groupname.\n";
	      }
	  }
	}
    }
    
    # now add anyone left in the userhash

    @addmembers = keys %userhash;

    if ($#addmembers >= 0)
      {
	$debug && print "Attempting to add @addmembers to group $groupname.\n";

	foreach $newuser (@addmembers) {
	  if (Win32::NetAdmin::GroupAddUsers($server, $groupname, $newuser)) {
	    $debug && print "Successfully added $newuser to group $groupname\n";
	  } else {
	    $debug && print "Could not add $newuser to group $groupname.\n";
	    print OUT "Could not add $newuser to group $groupname.\n";
	  }
	}
      }

#    if ($#killmembers == -1 && $#addmembers == -1)
#      {
#	$debug && print "Group $groupname not changing\n";
#      }
  } else {
    
    # create the group
    
    if (Win32::NetAdmin::GroupCreate($server, $groupname, $description)) {
      $debug && print "Successfully created group $groupname.\n";
      print OUT "Successfully created group $groupname.\n";
    } else {
      $debug && print "Could not successfully create group $groupname.\n";
      print OUT "Could not successfully create group $groupname.\n";
      return 0;
    }
    
    # add all the users to the group
    
    if ($#members > 0) {

      $debug && print "Attempting to add @members to group $groupname.\n";

      foreach $newuser (@members) {
	if (Win32::NetAdmin::GroupAddUsers($server, $groupname, $newuser)) {
	  $debug && print "Successfully added $newuser to group $groupname\n";
	} else {
	  $debug && print "Could not add $newuser to group $groupname.\n";
	  print OUT "Could not add $newuser to group $groupname.\n";
	}
      }
    }
  }
  
  return 1;
}

######################################################################
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub deleteGroup {
  my ($groupname) = @_;
  my $comment;

  if (Win32::NetAdmin::GroupGetAttributes($server, $groupname, $comment)) {
    if ($comment ne "Managed by Ganymede") {
      $debug && print "Group $groupname was not created by Ganymede.. will not delete.\n";
    }

    return 0;
  }

  if (Win32::NetAdmin::GroupDelete($server, $groupname)) {
    # create event log
    LogEvent ("delete","4","$groupname deleted");
    $debug && print "Group $groupname successfully deleted.\n";
    print OUT "Group $groupname successfully deleted.\n";
    return 1;
  } else {
    $debug && print "Group $groupname could not be successfully deleted.\n";
    print OUT "Group $groupname could not be successfully deleted.\n";
    return 0;
  }
}

######################################################################
#
# Returns 1 if success, 0 if failure
#
######################################################################
sub emptyGroup {
  my ($groupname) = @_;
  my @oldmembers, $person, $success;

  $success = 1;
  # create event log
  #  LogEvent ("delete","4","$groupname emptied/inactivated");

  Win32::NetAdmin::GroupGetMembers($server, $groupname, \@oldmembers);
  
  foreach $person (@oldmembers) {

    $debug && print "** Attempting to delete user $person from group $groupname.\n";
    if (Win32::NetAdmin::GroupDeleteUsers($server, $groupname, $person))
      {
	$debug && print "Successfully removed $person from inactive group $groupname.\n";
      }
    else
      {
	$debug && print "Could not remove $person from inactive group $groupname.\n";
	print OUT "Could not remove $person from inactive group $groupname.\n";
	$success = 0;
      }
  }

  if ($success) {
    $debug && print "Group $groupname was successfully emptied.\n";
    print OUT "Group $groupname was successfully emptied.\n";
    return 1;
  }

  return 0;
}

######################################################################
#
# Log an event in the event log
#
######################################################################
sub LogEvent {
## SEE: http://www.inforoute.cgs.fr/leberre1/main.htm for details on the eventlog.pm
	my ($category, $EventID, $msg) = @_;

	# open the event log.
	Win32::EventLog::Open($EventLog, "Ganymede", '');
        
	# define the event to log.
	$Event =
	{
		'Length'                => '',
                'RecordNumber'          => '',
                'TimeGenerated'         => '',
                'TimeWritten'           => '',
		'EventID' 		=> $EventID,
		'EventType' 		=> EVENTLOG_INFORMATION_TYPE,
		'Category' 		=> $category,
                'ClosingRecordNumber'   => '',
                'Source'                => '',
                'Computer'              => '',
		'Strings' 		=> $msg,
		'Data' 			=> '',
	};
	# report the event and check the error
	$EventLog->Report($Event);
	return;
}

######################################################################
#
# Subroutine to break a colon-separated line into fields. Supports
# backslashed-colons within fields.
#
######################################################################

sub colonsplitter {
    my ($string) = @_;
    my ($temp, @results, $i, $char);

    for ($i = 0; $i < length $string; $i++) {
      $char = substr($string, $i, 1);

      if ($char eq "\\") {
	$i++;
	$char = substr($string, $i, 1);
	
	$temp = $temp . $char;
      } elsif ($char eq ":") {
	push @results, $temp;
	$temp = "";
      } else {
	$temp = $temp . $char;
      }
    }

    chomp $temp;

    if ($temp ne "") {
      push @results, $temp;
    }
    
    return @results;
}
