##!<#PERLEXE#>
#
# This Perl script is installed by the configurator.perl script, and
# is responsible for integrating this schema kit into an installed
# Ganymede server.
#
###############################################################################

die "We require Perl 5.003 or greater to install Ganymede schema kit." if $] < 5.003;

use File::Copy;
use Cwd;
use English;

$schemaName = "userKit";

#
#
#

if (defined $ENV{GPERL}) {
  $perlname = $ENV{GPERL};
} else {
  die "Error, GPERL environment variable not set.  This script is not meant to be run by hand.  Use installKit instead.";
}

$sourcedir = &canonify($ENV{GROOTDIR});

# This script needs to get information from
# the user, including:
#
# Where is the Ganymede server installed at?
# Where are the java binaries located at?

@steps = ("SERVERINSTALL", "ANTPATH");

$antguess = `which ant`;
chop $antguess;
%vars = ("SERVERINSTALL" => "",
	 "ANTPATH" => $antguess);

%descrip = ("SERVERINSTALL" => "Where is the Ganymede server installed at?",
	    "ANTPATH" => "Where is ant located at?");

foreach $step (@steps) {

  $success = 0;

  while (!$success) {
    print $descrip{$step};
    print " [$vars{$step}] ";

    $inp = <>;

    chop $inp; # cut off newline

    if ($inp ne "") {
      $vars{$step} = $inp;
    }

    if ($vars{$step}eq "") {
      $success = 0;
      print "*You must provide a value for this field*\n";
      next;
    } else {
      $success = 1;
    }

    if ($step eq "SERVERINSTALL") {
      $installdir = &canonify($vars{"SERVERINSTALL"});
    }

    if ($step eq "ANTPATH") {
      $antpath = &canonify($vars{"ANTPATH"});
    }
  }
}

###
### Let's do it, then.
###

# 1. Things to do
#
# a. Check to see if the server directory exists, and whether it has
#    a 'schema' directory under it, in which case we surrender.
#
# b. Create a schema directory under the server

if (!-d $installdir) {
  die "Error, couldn't find $installdir.\n";
} else {
  if (-e "$installdir/schema") {
    if (-d "$installdir/schema") {
      die "Error, schema directory already exists under server install directory.\n";
    } else {
      die "Error, can't create the schema directory, already a schema file or link there.\n";
    }
  }

  &makedir("$installdir/schema", 0750);
}

# 2. Run ant on the source tree

if (!-d $installdir) {
  die "Error, couldn't find ganymede jars directory in $installdir.\n";
} else {
  system("$antpath -f $sourcedir/src/build.xml -Dganyjars.dir=$installdir/jars jars");
}

# 3. Copy our scripts and src trees under the schema target dir,
#    generating config.sh and rebuild files for the src tree,
#    from files in scripts

@dirsToCopy = ("src", "scripts", "Modules", "NTscripts");

foreach $dir (@dirsToCopy) {
  if (-d "$sourcedir/$dir") {
    &copydir("$sourcedir/$dir", "$installdir/schema/$dir");

    if ($dir eq "src") {
      &write_config("$installdir/schema/src", 
		    "Ganymede $schemaName", 
		    "$installdir/schema/src/classes",
		    "$installdir/jars/ganymedeServer.jar:$installdir/schema/src/classes");

      &write_rebuild("$installdir/schema/src");
    }
  }
}

# 4. Copy custom.jar, schema.xml, and any README files
#    Note that we are not copying installKit, since there's
#    no point trying to re-install the kit.  We do keep a
#    copy of installKit (installKit.in) in the scripts
#    directory, so makeArchive can regenerate it for
#    distribution

@filesToCopy = ("custom.jar", "schema.xml", "makeArchive", "README", "INSTALL", "COPYRIGHT");

foreach $file (@filesToCopy) {
  if (-f "$sourcedir/$file") {
    $mode = (stat("$sourcedir/$file"))[2] & 0777;
    copy ("$sourcedir/$file", "$installdir/schema/$file");
    chmod($mode, "$installdir/schema/$file");
  }
}

# 5. Create output directory under schema, as necessary
#    for the operation of our scripts

&makedir("$installdir/schema/output", 0750);
&makedir("$installdir/schema/output/unix", 0750);
&makedir("$installdir/schema/output/bsd", 0750);
&makedir("$installdir/schema/output/ntsamba", 0750);

# 6. Copy and configure 'loadSchema' and 'loadData' scripts.. do
#    we want these to be separate scripts, or do we want just one?
#    Should the goal of these scripts be to simply create an xml file
#    to be loaded by the user manually using xmlclient once the server
#    is started?
#
# Essentially, we want to get to a point where all we need to retain
# for packaging is the contents of scripts and src, schema.xml, custom.jar,
# configure, and any README files.  It will be up to installKit to reconstitute a
# working environment under schema the next time the schema kit is merged
# with a server install.

$templates = {"scripts/install-modules.pl.in" => "install-modules.pl",
	      "scripts/loader.pl.in" => "loader.pl",
	      "scripts/newUserScript.in" => "output/newUserScript",
	      "scripts/deleteUserScript.in" => "output/deleteUserScript",
	      "scripts/renameUserScript.in" => "output/renameUserScript",
	      "scripts/bsdBuild.in" => "output/bsdBuild",
	      "scripts/unixBuild.in" => "output/unixBuild",
	      "scripts/ntSambaBuild.in" => "output/ntSambaBuild",
	      "scripts/ntSync" => "output/ntSync",
	      "scripts/sambaSync" => "output/sambaSync"};

foreach $file (keys %{$templates}) {
  $target = $templates->{$file};
  $hashref = {"<\#PERLEXE\#>" => "$perlname",
	      "<\#SCHEMAROOT\#>" => "$installdir/schema",
	      "<\#MODULESDIR\#>" => "$installdir/schema/Modules"};

  &installtemplate("$sourcedir/$file", "$installdir/schema/$target", $hashref);
}

print <<DONE;
This schema kit has been successfully integrated with your Ganymede
server installation at $installdir.

To continue with schema installation, you should change directory to
$installdir/schema and run the loader.pl script there to configure
the schema kit and to import your local directory data.

Once you have run loader.pl and generated the data.xml file that it
produces, you should start your server and run xmlclient on the
data.xml file to load everything into the running server.  See
$installdir/schema/README for complete details.
DONE

#########################################################################
#
#                                                            write_config
#
# input: $dir - absolute pathname directory to write config.sh into.
#        $name - name of component being configured
#        $targetdir - where is the class directory for this package?
#        $classpath - CLASSPATH for building source after install
#
#########################################################################

sub write_config {

    my($dir, $name, $targetdir, $classpath) = @_;

    open(CONFIGFILE, ">$dir/config.sh") || die("Can't create the $dir/config.sh");

    print CONFIGFILE <<ENDCONFIG2;
#
# $name config.sh
#
# Auto-generated by installKit
#

# Name of component package being built

COMPNAME="$name"

# Location of java compiler

JAVAC=$javadir/javac

# Location of directory containing java executables

JAVADIR=$javadir

# Master location for ganymede classes.. the java compiler
# will look here to find pre-compiled classes needed to compile
# code in this directory

CLASSDIR=$classpath

# Target location for classes built from sources in this directory.
# This is just the root of the tree.. the javac compiler will
# place classes in a subdirectory of this directory according to
# the package name

TARGETDIR=$targetdir

ENDCONFIG2

    close(CONFIGFILE);
}

#########################################################################
#
#                                                           write_rebuild
#
# input: $dir - absolute pathname directory to write rebuild script into.
#
#########################################################################

sub write_rebuild {

    my ($dir) = @_;

    open(REBUILDIN, "<$sourcedir/scripts/rebuild.in") || die ("Can't read $sourcedir/scripts/rebuild.in");
    open(REBUILDOUT, ">$dir/rebuild") || die("Can't create the $dir/rebuild");

    while (<REBUILDIN>){
	s/\<\#ANTPATH\#\>/$antpath/g;
	print REBUILDOUT $_;
    }

    close(REBUILDOUT);
    close(REBUILDIN);

    chmod 0755, "$dir/rebuild";
}

#########################################################################
#
#                                                                 resolve
#
# input: $dir - absolute pathname of current directory
#        $link - string containing the readlink() results for a 
#                symbolic link in $dir to be processed
#
# returns: absolute pathname of the target of the symbolic link
#
#########################################################################
sub resolve{
    my($dir, $link) = @_;
    my(@alinkp, $d, $alinkp);

    # make array representations of
    # the current directory and symbolic link

    # if we have a leading / in our $dir or $link,
    # we'll need to shift to get rid of the leading
    # empty array element

    @dirp=split(/\//, $dir);
    shift(@dirp) if (!($dirp[0]));
    @linkp=split(/\//, $link);
    shift(@linkp) if (!($linkp[0]));

    # @alinkp is an array that we will build to contain the absolute link
    # target pathname.  If the link does not begin with a /, it is a relative link,
    # and we need to place our current directory into the @alinkp array.  

    if ($link !~ /^\//) {
	@alinkp=@dirp;
    }

    # modify the @alinkp array according
    # to each path component of the @linkp array
    # (an array representation of the symbolic link
    # given to us), to arrive at the ultimate absolute
    # pathname of the symbolic link

    $d = shift(@linkp);

    while ($d) {
	if ($d eq "..") {
	    pop(@alinkp);
	}
	elsif ($d ne "."){
	    push(@alinkp, $d);
	}
	$d=shift(@linkp);
    }

    $"='/';

    # perl functions return the value of the last expression
    # in the subroutine

    $alinkp="/@alinkp";
}

#########################################################################
#
#                                                                canonify
#
# input: $pathname - a directory or filename path, may be relative
# or use ~ to represent the current user's home directory
#
# returns: absolute location of the givevn pathname
#
#########################################################################
sub canonify {
  my ($pathname) = @_;
  my ($homedir);

  if (!(($pathname =~ /^\//) || ($pathname =~ /^~\//))) {
    $pathname = &resolve(cwd(), $pathname);
  }

  if ($pathname =~ /^~\//) {
    $homedir = $ENV{'HOME'};
    $pathname =~ s/\~\//$homedir\//g;
  }

  return $pathname;
}

######################################################################### 
#
#                                                         removelastslash
#
# input: a pathname to test
#
# this function will remove a trailing slash from the directory name 
# input
#
######################################################################### 
sub removelastslash{
  if ($_[0] =~ /\/$/) {
    chop $_[0];
  }

  return $_[0];
}

######################################################################### 
#
#                                                                 dirname
#
# input: 1) a filename path
#
# return: the directory containing the filename given
#
######################################################################### 
sub dirname{
  my ($dirpath) = @_;
  my (@comps);

  @comps = split(/\//, $dirpath);
  pop @comps;
  return join('/', @comps);
}

######################################################################### 
#
#                                                                 makedir
#
# input: 1) a directory to make
#        2) octal chmod bits
#
######################################################################### 
sub makedir{
  my ($dirpath, $chmod) = @_;

  if (!-e $dirpath) {
    mkdir ($dirpath, $chmod) or die("*Couldn't make $dirpath*");
  }
}

######################################################################### 
#
#                                                                 copydir
#
# input: 1) a directory to copy from
#        2) directory target
#
######################################################################### 
sub copydir{
  my ($source, $target) = @_;
  my (@dirs, $file, $mode);

  &removelastslash($source);
  &removelastslash($target);

  if (!-e $target) {
    &makedir($target, 0750);
  }

  opendir SOURCE, $source || die "Failure in copydir";
  @dirs = readdir SOURCE;
  closedir SOURCE;

  foreach $file (@dirs) {
    if (($file eq ".") || ($file eq "..")) {
      next;
    }

    if (-d "$source/$file") {
      &copydir("$source/$file", "$target/$file"); #recurse
    } else {
      $mode = (stat("$source/$file"))[2] & 0777;

      if (($file =~ /\.pl$/) || ($file =~ /\.perl$/)) {

	# we've got a perl script, copy it and fix up the ##!<#PERLEXE#> line
	# along the way

	open(COPYFILEIN, "<$source/$file") || die ("Can't read $source/$file");
	open(COPYFILEOUT, ">$target/$file") || die("Can't create the $target/file");
	
	while (<COPYFILEIN>){
	  s/^\#\#!<\#PERLEXE\#\>/#!$perlname/g;
	  print COPYFILEOUT $_;
	}
	
	close(COPYFILEOUT);
	close(COPYFILEIN);
      } else {
	copy("$source/$file", "$target/$file");
      }
      chmod($mode, "$target/$file");
    }
  }
}

######################################################################### 
#
#                                                         installtemplate
#
# input: 1) the template file to read from
#        2) the file to write to
#        3) an optional hashref of regexp patterns to modify, with the
#           keys being the regexp to match and the value the replacement
#           text
#
######################################################################### 
sub installtemplate {
  my ($source, $target, $regexps) = @_;
  my ($mode, $key, $value);

  if (!-e $source) {
    print "Error, bad file $source requested of installtemplate\n";
    return;
  }

  open(SOURCETEMPLATE, "<$source") || die ("Can't read $source");
  open(TEMPLATETARGET, ">$target") || die ("Can't open $target");

  $mode = (stat("$source"))[2] & 0777;

  while (<SOURCETEMPLATE>) {
    foreach $key (keys %{$regexps}) {
      $value = $regexps->{$key};
      s/$key/$value/g;
    }

    print TEMPLATETARGET $_;
  }

  close SOURCETEMPLATE;
  close TEMPLATETARGET;

  chmod($mode, "$target");
}
